@@ -182,9 +182,11 @@ void
 environ_update(struct options *oo, struct environ *src, struct environ *dst)
 {
 	struct environ_entry		*envent;
+	struct environ_entry		*envent1;
 	struct options_entry		*o;
 	struct options_array_item	*a;
 	union options_value		*ov;
+	int				 found;
 
 	o = options_get(oo, "update-environment");
 	if (o == NULL)
@@ -192,14 +194,15 @@ environ_update(struct options *oo, struct environ *src, struct environ *dst)
 	a = options_array_first(o);
 	while (a != NULL) {
 		ov = options_array_item_value(a);
-		RB_FOREACH(envent, environ, src) {
-			if (fnmatch(ov->string, envent->name, 0) == 0)
-				break;
+		found = 0;
+		RB_FOREACH_SAFE(envent, environ, src, envent1) {
+			if (fnmatch(ov->string, envent->name, 0) == 0) {
+				environ_set(dst, envent->name, 0, "%s", envent->value);
+				found = 1;
+			}
 		}
-		if (envent == NULL)
+		if (!found)
 			environ_clear(dst, ov->string);
-		else
-			environ_set(dst, envent->name, 0, "%s", envent->value);
 		a = options_array_next(a);
 	}
 }
@@ -261,6 +264,12 @@ environ_for_session(struct session *s, int no_TERM)
 		environ_set(env, "TERM_PROGRAM_VERSION", 0, "%s", getversion());
 	}
 
+#ifdef HAVE_SYSTEMD
+	environ_clear(env, "LISTEN_PID");
+	environ_clear(env, "LISTEN_FDS");
+	environ_clear(env, "LISTEN_FDNAMES");
+#endif
+
 	if (s != NULL)
 		idx = s->id;
 	else
