@@ -335,7 +335,7 @@ menu_key_cb(struct client *c, void *data, struct key_event *event)
 		c->flags |= CLIENT_REDRAWOVERLAY;
 		return (0);
 	case KEYC_PPAGE:
-	case '\002': /* C-b */
+	case 'b'|KEYC_CTRL:
 		if (md->choice < 6)
 			md->choice = 0;
 		else {
@@ -394,13 +394,13 @@ menu_key_cb(struct client *c, void *data, struct key_event *event)
 		}
 		c->flags |= CLIENT_REDRAWOVERLAY;
 		break;
-	case '\006': /* C-f */
+	case 'f'|KEYC_CTRL:
 		break;
 	case '\r':
 		goto chosen;
 	case '\033': /* Escape */
-	case '\003': /* C-c */
-	case '\007': /* C-g */
+	case 'c'|KEYC_CTRL:
+	case 'g'|KEYC_CTRL:
 	case 'q':
 		return (1);
 	}
