@@ -22,7 +22,6 @@
 #include <sys/sysctl.h>
 
 #include <errno.h>
-#include <event.h>
 #include <limits.h>
 #include <stdlib.h>
 #include <string.h>
@@ -36,9 +35,6 @@
 	((p)->p_stat == SSTOP || (p)->p_stat == SZOMB)
 
 struct kinfo_proc2	*cmp_procs(struct kinfo_proc2 *, struct kinfo_proc2 *);
-char			*osdep_get_name(int, char *);
-char			*osdep_get_cwd(int);
-struct event_base	*osdep_event_init(void);
 
 struct kinfo_proc2 *
 cmp_procs(struct kinfo_proc2 *p1, struct kinfo_proc2 *p2)
